﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.Interface;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase
{
    [ApiDescriptionSettings("Interface", Tag = "接口信息")]
    public class InterfaceServices : IDynamicApiController
    {
        IInterfaceService _interfaceService;
        public InterfaceServices(IInterfaceService interfaceService)
        {
            _interfaceService = interfaceService;
        }

        #region 获取信息
        #region 账号
        /// <summary>
        /// 获取所有账号信息
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/interface/getUsers")]
        public async Task<List<UsersDto>> GetUsers()
        {
            return await _interfaceService.GetUsers();
        }
        #endregion

        #region 设备autokey获取信息
        /// <summary>
        /// 获取设备信息
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/interface/getDeviceInfo")]
        public async Task<DeviceInfoDto> GetDeviceInfo()
        {
            return await _interfaceService.GetDeviceInfo();
        }
        #endregion

        #region 获取配方信息
        /// <summary>
        /// 获取所有配方信息
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/interface/getRecipeList")]
        public async Task<RecipeDto> GetRecipeList()
        {
            return await _interfaceService.GetRecipeList();
        }
        #endregion
        #endregion

        #region 物料
        /// <summary>
        /// 获取所有物料列表
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/interface/getBatching")]
        public async Task<List<BatchingDto>> GetBatching()
        {
            return await _interfaceService.GetBatching();
        }

        /// <summary>
        /// 上传更新物料信息
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/uploadBatching")]
        public async Task<List<InterfaceMessageDto>> UploadBatching(List<BatchingDataDto> dto)
        {
            return await _interfaceService.UploadBatching(dto);
        }

        /// <summary>
        /// 删除物料
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/deleteBatching")]
        public async Task<List<InterfaceMessageDto>> DeleteBatching(List<BatchingDelDto> dto)
        {
            return await _interfaceService.DeleteBatching(dto);
        }
        #endregion

        #region 配方

        #region 配方信息
        ///// <summary>
        ///// 获取分组下配方信息
        ///// </summary>
        ///// <param name="recipeGroupId"></param>
        ///// <returns></returns>
        //[HttpGet("/api/interface/getRecipeInfo")]
        //public async Task<List<RecipeInfoDto>> GetRecipeInfo(string recipeGroupId)
        //{
        //    return await _interfaceService.GetRecipeInfo(recipeGroupId);
        //}

        /// <summary>
        /// 上传更新配方信息
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/uploadRecipeInfo")]
        public async Task<List<InterfaceMessageDto>> UploadRecipeInfo(List<RecipeInfoDataDto> dto)
        {
            return await _interfaceService.UploadRecipeInfo(dto);
        }

        ///// <summary>
        ///// 删除配方
        ///// </summary>
        ///// <param name="dto"></param>
        ///// <returns></returns>
        //[HttpPost("/api/interface/deleteRecipeInfo")]
        //public async Task<List<InterfaceMessageDto>> DeleteRecipeInfo(List<RecipeInfoDelDto> dto)
        //{
        //    return await _interfaceService.DeleteRecipeInfo(dto);
        //}
        #endregion

        #region 配方版本
        /// <summary>
        /// 上传更新配方版本
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/uploadRecipeVersion")]
        public async Task<List<InterfaceMessageDto>> UploadRecipeVersion(List<RecipeVersionDataDto> dto)
        {
            return await _interfaceService.UploadRecipeVersion(dto);
        }
        #endregion

        #region 配方多属性
        ///// <summary>
        ///// 获取配方多属性
        ///// </summary>
        ///// <param name="recipeId"></param>
        ///// <returns></returns>
        //[HttpGet("/api/interface/getRecipeAttr")]
        //public async Task<List<RecipeAttrDto>> GetRecipeAttr(string recipeId)
        //{
        //    return await _interfaceService.GetRecipeAttr(recipeId);
        //}

        /// <summary>
        /// 上传更新配方多属性
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/uploadRecipeAttr")]
        public async Task<List<InterfaceMessageDto>> UploadRecipeAttr(List<RecipeAttrDataDto> dto)
        {
            return await _interfaceService.UploadRecipeAttr(dto);
        }

        ///// <summary>
        ///// 删除配方多属性
        ///// </summary>
        ///// <param name="dto"></param>
        ///// <returns></returns>
        //[HttpPost("/api/interface/deleteRecipeAttr")]
        //public async Task<List<InterfaceMessageDto>> DeleteRecipeAttr(List<AttrValueDelDto> dto)
        //{
        //    return await _interfaceService.DeleteRecipeAttr(dto);
        //}
        #endregion

        #region 配方物料
        ///// <summary>
        ///// 获取配方物料
        ///// </summary>
        ///// <param name="recipeId"></param>
        ///// <returns></returns>
        //[HttpGet("/api/interface/getRecipeBatching")]
        //public async Task<List<RecipeBatchingDto>> GetRecipeBatching(string recipeId)
        //{
        //    return await _interfaceService.GetRecipeBatching(recipeId);
        //}

        /// <summary>
        /// 上传更新配方物料
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/uploadRecipeBatching")]
        public async Task<List<InterfaceMessageDto>> UploadRecipeBatching(List<RecipeBatchingDataDto> dto)
        {
            return await _interfaceService.UploadRecipeBatching(dto);
        }

        ///// <summary>
        ///// 删除配方物料
        ///// </summary>
        ///// <param name="dto"></param>
        ///// <returns></returns>
        //[HttpPost("/api/interface/deleteRecipeBatching")]
        //public async Task<List<InterfaceMessageDto>> DeleteRecipeBatching(List<RecipeBatchingDelDto> dto)
        //{
        //    return await _interfaceService.DeleteRecipeBatching(dto);
        //}
        #endregion

        #region 配方模版
        /// <summary>
        /// 上传更新配方模版
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/uploadRecipeTemplate")]
        public async Task<List<InterfaceMessageDto>> UploadRecipeTemplate(List<RecipeTemplateDataDto> dto)
        {
            return await _interfaceService.UploadRecipeTemplate(dto);
        }
        #endregion

        #endregion

        #region 报表接口
        /// <summary>
        /// 上传配方使用数据
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/uploadRecipeUseDetail")]
        public async Task<List<InterfaceMessageDto>> UploadRecipeUseDetail(List<RecipeUseDetailDataDto> dto)
        {
            return await _interfaceService.UploadRecipeUseDetail(dto);
        }
        #endregion

        #region 日志接口
        /// <summary>
        /// 上传设备日志
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/uploadDeviceLog")]
        public async Task<List<InterfaceMessageDto>> UploadDeviceLog([FromForm] DeviceLogDto dto)
        {
            return await _interfaceService.UploadDeviceLog(dto);
        }
        #endregion

        #region 激活接口
        /// <summary>
        /// 在线激活码激活设备
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/activationDevice")]
        public async Task<List<InterfaceMessageDto>> ActivationDevice(ActivationDeviceDto dto)
        {
            return await _interfaceService.ActivationDevice(dto);
        }

        /// <summary>
        /// 上传离线激活记录
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/interface/uploadOfflineLog")]
        public async Task<List<InterfaceMessageDto>> UploadOfflineLog(OfflineActivationLogDto dto)
        {
            return await _interfaceService.UploadOfflineLog(dto);
        }
        #endregion
    }
}
