﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.Recipe;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Core.Base;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase
{
    [ApiDescriptionSettings("Recipe", Tag = "配方信息")]
    public class RecipeServices : IDynamicApiController
    {
        IRecipeService _recipeService;
        public RecipeServices(IRecipeService recipeService)
        {
            _recipeService = recipeService;
        }
        #region 配方
        /// <summary>
        /// 获取配方分页列表
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/recipe/getRecipeInfoPage")]
        public async Task<PageUtil> GetRecipeInfoPage(RecipeQueryDto dto)
        {
            return await _recipeService.GetRecipeInfoPage(dto);
        }

        /// <summary>
        /// 更新配方
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/recipe/updateRecipeInfo")]
        public async Task<bool> UpdateRecipeInfo(RecipeResUpDto dto)
        {
            return await _recipeService.UpdateRecipeInfo(dto);
        }

        /// <summary>
        /// 删除配方
        /// </summary>
        /// <param name="ids"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/deleteRecipeInfo")]
        public async Task<bool> DeleteRecipeInfo(string ids, int autoKey)
        {
            return await _recipeService.DeleteRecipeInfo(ids, autoKey);
        }

        /// <summary>
        /// 根据配方Id获取配方版本
        /// </summary>
        /// <param name="recipeId"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getRecipeVersionList")]
        public async Task<List<RecipeVersionResDto>> GetRecipeVersionList(string recipeId, int autoKey)
        {
            return await _recipeService.GetRecipeVersionList(recipeId, autoKey);
        }

        /// <summary>
        /// 更新配方版本
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/recipe/updateRecipeVersion")]
        public async Task<bool> UpdateRecipeVersion(RecipeVersionResUpDto dto)
        {
            return await _recipeService.UpdateRecipeVersion(dto);
        }

        /// <summary>
        /// 删除配方版本
        /// </summary>
        /// <param name="ids"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/deleteRecipeVersion")]
        public async Task<bool> DeleteRecipeVersion(string ids, int autoKey)
        {
            return await _recipeService.DeleteRecipeVersion(ids, autoKey);
        }

        /// <summary>
        /// 获取基本配方详情
        /// </summary>
        /// <param name="recipeVersionId"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getBaseBatchingList")]
        public async Task<List<RecipeBatchingDto>> GetBaseBatchingList(string recipeVersionId, int autoKey)
        {
            return await _recipeService.GetBaseBatchingList(recipeVersionId, autoKey);
        }

        /// <summary>
        /// 获取多属性配方详情
        /// </summary>
        /// <param name="recipeVersionId"></param>
        /// <param name="attrIds"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getAttrBatchingList")]
        public async Task<List<RecipeBatchingDto>> GetAttrBatchingList(string recipeVersionId, string attrIds, int autoKey)
        {
            return await _recipeService.GetAttrBatchingList(recipeVersionId, attrIds, autoKey);
        }

        /// <summary>
        /// 设置配方物料
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/recipe/setRecipeBatching")]
        public async Task<bool> SetRecipeBatching(RecipeBatchingDataDto dto)
        {
            return await _recipeService.SetRecipeBatching(dto);
        }

        /// <summary>
        /// 获取已配置多属性配方列表
        /// </summary>
        /// <param name="recipeVersionId"></param>
        /// <param name="valueIds"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getAttrRecipeList")]
        public async Task<List<AttrRecipeListDto>> GetAttrRecipeList(string recipeVersionId, string valueIds, int autoKey)
        {
            return await _recipeService.GetAttrRecipeList(recipeVersionId, valueIds, autoKey);
        }

        /// <summary>
        /// 删除多属性物料列表
        /// </summary>
        /// <param name="configId"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/deleteRecipeAttr")]
        public async Task<bool> DeleteRecipeAttr(string configId, int autoKey)
        {
            return await _recipeService.DeleteRecipeAttr(configId, autoKey);
        }
        #endregion
        /// <summary>
        /// 获取配方分组
        /// </summary>
        /// <param name="groupTempId"></param>
        /// <param name="isWeight"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getRecipeGroupList")]
        public async Task<List<RecipeGroupDto>> GetRecipeGroupList(string groupTempId, bool isWeight, int autoKey)
        {
            return await _recipeService.GetRecipeGroupList(groupTempId ,isWeight, autoKey);
        }

        /// <summary>
        /// 更新配方分组
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/recipe/updataRecipeGroup")]
        public async Task<bool> UpdataRecipeGroup(RecipeGroupDto dto)
        {
            return await _recipeService.UpdataRecipeGroup(dto);
        }

        /// <summary>
        /// 删除配方分组
        /// </summary>
        /// <param name="id"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/deleteRecipeGroup")]
        public async Task<bool> DeleteRecipeGroup(string id, int autoKey)
        {
            return await _recipeService.DeleteRecipeGroup(id, autoKey);
        }

        /// <summary>
        /// 通过配方分组获取配方最大数
        /// </summary>
        /// <param name="recipeGroupId"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getRecipeTemplateCount")]
        public async Task<int> GetRecipeTemplateCount(string recipeGroupId, int autoKey)
        {
            return await _recipeService.GetRecipeTemplateCount(recipeGroupId, autoKey);
        }

        /// <summary>
        /// 通过配方分组和页数获取配方信息
        /// </summary>
        /// <param name="recipeGroupId"></param>
        /// <param name="page"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getRecipeTemplateList")]
        public async Task<List<RecipeTemplateInfoDto>> GetRecipeTemplateList(string recipeGroupId, int page, int autoKey)
        {
            return await _recipeService.GetRecipeTemplateList(recipeGroupId, page, autoKey);
        }

        /// <summary>
        /// 更新绑定模版
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/recipe/updateRecipeTemplate")]
        public async Task<bool> UpdateRecipeTemplate(RecipeTemplateUpDto dto)
        {
            return await _recipeService.UpdateRecipeTemplate(dto);
        }

        /// <summary>
        /// 删除绑定模版
        /// </summary>
        /// <param name="id"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/deleteRecipeTemplate")]
        public async Task<bool> DeleteRecipeTemplate(string id, int autoKey)
        {
            return await _recipeService.DeleteRecipeTemplate(id, autoKey);
        }

        /// <summary>
        /// 增加一页配方
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/recipe/addRecipeTemplatePage")]
        public async Task<bool> AddRecipeTemplatePage(RecipePageDto dto)
        {
            return await _recipeService.AddRecipeTemplatePage(dto);
        }

        /// <summary>
        /// 删除当前页所有配方
        /// </summary>
        /// <param name="recipeGroupId"></param>
        /// <param name="page"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/deleteRecipeTemplatePage")]
        public async Task<bool> DeleteRecipeTemplatePage(string recipeGroupId, int page, int autoKey)
        {
            return await _recipeService.DeleteRecipeTemplatePage(recipeGroupId, page, autoKey);
        }

        /// <summary>
        /// 获取配方和版本列表
        /// </summary>
        /// <param name="recipeId"></param>
        /// <param name="isWeight"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getRecipeVersionSelectList")]
        public async Task<List<RecipeSelectDto>> GetRecipeVersionSelectList(string recipeId, bool isWeight, int autoKey)
        {
            return await _recipeService.GetRecipeVersionSelectList(recipeId ,isWeight, autoKey);
        }

        /// <summary>
        /// 获取配方最大版本数
        /// </summary>
        /// <param name="recipeId"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getMaxVersion")]
        public async Task<int> GetMaxVersion(string recipeId, int autoKey)
        {
            return await _recipeService.GetMaxVersion(recipeId, autoKey);
        }

        /// <summary>
        /// 获取分组最大序号数
        /// </summary>
        /// <param name="isWeight"></param>
        /// <param name="autoKey"></param>
        /// <param name="groupTempId"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getGroupMaxSort")]
        public async Task<int> GetGroupMaxSort(bool isWeight, int autoKey, string groupTempId)
        {
            return await _recipeService.GetGroupMaxSort(isWeight, autoKey, groupTempId);
        }

        /// <summary>
        /// 获取复制列表
        /// </summary>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getCopyList")]
        public async Task<List<CopyItemDto>> GetCopyList(int autoKey)
        {
            return await _recipeService.GetCopyList(autoKey);
        }

        /// <summary>
        /// 获取所有配方
        /// </summary>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getAllRecipeList")]
        public async Task<List<RecipeResDto>> GetAllRecipeList(int autoKey)
        {
            return await _recipeService.GetAllRecipeList(autoKey);
        }

        /// <summary>
        /// 获取模版
        /// </summary>
        /// <param name="select"></param>
        /// <param name="autoKey"></param>
        /// <param name="groupTempId"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getTemplateList")]
        public async Task<List<RecipeGroupTemplateDto>> GetTemplateList(bool select, int autoKey, string groupTempId)
        {
            return await _recipeService.GetTemplateList(select, autoKey, groupTempId);
        }

        /// <summary>
        /// 根据名字获取配方
        /// </summary>
        /// <param name="name"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getRecipeListWithName")]
        public async Task<List<RecipeResDto>> GetRecipeListWithName(string name, int autoKey)
        {
            return await _recipeService.GetRecipeListWithName(name, autoKey);
        }

        /// <summary>
        /// 获取重复配方
        /// </summary>
        /// <param name="names"></param>
        /// <param name="selectAutoKey"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/getRepeatRecipeList")]
        public async Task<List<RecipeResDto>> GetRepeatRecipeList(string names, int selectAutoKey, int autoKey)
        {
            return await _recipeService.GetRepeatRecipeList(names, selectAutoKey, autoKey);
        }

        /// <summary>
        /// 复制配方
        /// </summary>
        /// <param name="names"></param>
        /// <param name="skipNames"></param>
        /// <param name="newAutoKey"></param>
        /// <param name="oldAutoKey"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/copyRecipeList")]
        public async Task<bool> CopyRecipeList(string names, string skipNames, int newAutoKey, int oldAutoKey)
        {
            return await _recipeService.CopyRecipeList(names, skipNames, newAutoKey, oldAutoKey);
        }

        /// <summary>
        /// 复制配方模版
        /// </summary>
        /// <param name="newAutoKey"></param>
        /// <param name="oldAutoKey"></param>
        /// <param name="groupTempId"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/copyTemplateList")]
        public async Task<bool> CopyTemplateList(string newAutoKey, int oldAutoKey, string groupTempId)
        {
            return await _recipeService.CopyTemplateList(newAutoKey, oldAutoKey, groupTempId);
        }

        /// <summary>
        /// 获取配方模版列表
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/recipe/getRecipeGroupTemplatePage")]
        public async Task<PageUtil> GetRecipeGroupTemplatePage(RecipeGroupTemplateQueryDto dto)
        {
            return await _recipeService.GetRecipeGroupTemplatePage(dto);
        }

        /// <summary>
        /// 更新配方模版
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/recipe/updateRecipeGroupTemplate")]
        public async Task<bool> UpdateRecipeGroupTemplate(RecipeGroupTemplateUpDto dto)
        {
            return await _recipeService.UpdateRecipeGroupTemplate(dto);
        }

        /// <summary>
        /// 删除配方模版
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        [HttpGet("/api/recipe/deleteRecipeGroupTemplate")]
        public async Task<bool> DeleteRecipeGroupTemplate(string ids)
        {
            return await _recipeService.DeleteRecipeGroupTemplate(ids);
        }
    }
}
