﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.Report;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Core.Base;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase
{
    [ApiDescriptionSettings("Report", Tag = "报表信息")]
    public class ReportServices : IDynamicApiController
    {
        IReportService _reportService;
        public ReportServices(IReportService reportService)
        {
            _reportService = reportService;
        }

        /// <summary>
        /// 获取配方使用明细
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/report/getRecipeUseDetail")]
        public async Task<List<RecipeUseDetailDto>> GetRecipeUseDetail(RecipeQueryDto dto)
        {
            return await _reportService.GetRecipeUseDetail(dto);
        }

        /// <summary>
        /// 下载配方使用明细
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/report/downRecipeUseDetail")]
        public async Task<string> DownRecipeUseDetail(RecipeQueryDto dto)
        {
            return await _reportService.DownRecipeUseDetail(dto);
        }

        /// <summary>
        /// 获取配方使用汇总
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/report/getRecipeUseSummary")]
        public async Task<List<RecipeUseSummaryDto>> GetRecipeUseSummary(RecipeQueryDto dto)
        {
            return await _reportService.GetRecipeUseSummary(dto);
        }

        /// <summary>
        /// 下载配方使用汇总
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/report/downRecipeUseSummary")]
        public async Task<string> DownRecipeUseSummary(RecipeQueryDto dto)
        {
            return await _reportService.DownRecipeUseSummary(dto);
        }

        /// <summary>
        /// 物料使用汇总
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/report/getBatchingUseSummary")]
        public async Task<List<RecipeBatchingUseSummaryDto>> GetBatchingUseSummary(BatchingQueryDto dto)
        {
            return await _reportService.GetBatchingUseSummary(dto);
        }

        /// <summary>
        /// 下载物料使用汇总
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/report/downBatchingUseSummary")]
        public async Task<string> DownBatchingUseSummary(BatchingQueryDto dto)
        {
            return await _reportService.DownBatchingUseSummary(dto);
        }

        /// <summary>
        /// 获取下拉菜单
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/report/getSearchList")]
        public async Task<ListDto> GetSearchList()
        {
            return await _reportService.GetSearchList();
        }

        /// <summary>
        /// 获取首页统计
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/report/getMainReport")]
        public async Task<RankDto> GetMainReport(MainReportQueryDto dto)
        {
            return await _reportService.GetMainReport(dto);
        }

        /// <summary>
        /// 获取首页查询记录
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/report/getReportSearchLog")]
        public async Task<MainReportQueryDto> GetReportSearchLog()
        {
            return await _reportService.GetReportSearchLog();
        }
    }
}
