﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.Recipe;
using BPA.SAAS.Manage.Application.DataBase.Dtos.RecipeAttribute;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Org;
using BPA.SAAS.Manage.Core.Recipe;
using BPA.SAAS.Manage.Core.RecipeAttribute;
using BPA.SAAS.Manage.Core.Store;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class RecipeService : IRecipeService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public RecipeService(ISqlSugarClient db)
        {
            _db = db;
        }
        #region 配方
        public async Task<PageUtil> GetRecipeInfoPage(RecipeQueryDto dto)
        {
            var conModels = new List<IConditionalModel>();
            if (!string.IsNullOrEmpty(dto.Name))
                conModels.Add(new ConditionalModel() { FieldName = "Name", ConditionalType = ConditionalType.Like, FieldValue = dto.Name });
            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_RecipeInfo>()
                .Where(conModels)
                .Where(t => t.AutoKey == dto.AutoKey)
                .WhereIF(dto.IsWeight != null, x => x.IsWeight == dto.IsWeight)
                .OrderByDescending(t => t.CreateAt)
                .Select(t => new RecipeResDto
                {
                    Id = t.Id,
                    Name = t.Name,
                    IsWeight = t.IsWeight,
                    ReferenceWeight = t.ReferenceWeight,
                    CreateAt = t.CreateAt
                })
               .ToPageListAsync(dto.Current, dto.PageSize, total);
            var util = new PageUtil()
            {
                Total = total,
                Data = res
            };
            return util;
        }

        public async Task<bool> UpdateRecipeInfo(RecipeResUpDto dto)
        {
            var recipeList = await _db.Queryable<BPA_RecipeInfo>().ToListAsync();
            if (string.IsNullOrWhiteSpace(dto.Id))
            {
                var same = recipeList.Any(t => t.Name == dto.Name && t.AutoKey == dto.AutoKey);
                if (same)
                    throw Oops.Oh($"{dto.Name}已存在");
                var recipe = new BPA_RecipeInfo
                {
                    AutoKey = dto.AutoKey,
                    Name = dto.Name,
                    IsWeight = dto.IsWeight,
                    ReferenceWeight = dto.ReferenceWeight
                };
                await _db.Insertable(recipe).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
            }
            else
            {
                var recipe = recipeList.FirstOrDefault(t => t.Id == dto.Id && t.AutoKey == dto.AutoKey);
                if (recipe == null)
                    throw Oops.Oh($"{dto.Name}不存在或已删除");
                if (recipe.Name != dto.Name)
                {
                    var same = recipeList.Any(t => t.Name == dto.Name && t.AutoKey == dto.AutoKey);
                    if (same)
                        throw Oops.Oh($"{dto.Name}已存在");
                }
                recipe.Name = dto.Name;
                recipe.IsWeight = dto.IsWeight;
                recipe.ReferenceWeight = dto.ReferenceWeight;
                await _db.Updateable(recipe).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
            }
            return true;
        }

        public async Task<bool> DeleteRecipeInfo(string ids, int autoKey)
        {
            try
            {
                _db.Ado.BeginTran();
                var idList = ids.Split(',');
                var isUse = await _db.Queryable<BPA_RecipeTemplate>().AnyAsync(t => idList.Contains(t.RecipeId) && t.AutoKey == autoKey);
                if (isUse)
                    throw Oops.Oh("所选配方已被模版使用");
                var recipeList = await _db.Queryable<BPA_RecipeInfo>().Where(t => idList.Contains(t.Id) && t.AutoKey == autoKey).ToListAsync();
                var versionList = await _db.Queryable<BPA_RecipeVersion>().Where(t => idList.Contains(t.RecipeId) && t.AutoKey == autoKey).ToListAsync();
                var versionIdList = versionList.Select(t => t.Id).ToList();
                var attrList = await _db.Queryable<BPA_RecipeAttribute>().Where(t => versionIdList.Contains(t.RecipeVersionId) && t.AutoKey == autoKey).ToListAsync();
                var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => attrList.Select(a => a.Id).Contains(t.AttributeId) && t.AutoKey == autoKey).ToListAsync();
                var baseConfigList = await _db.Queryable<BPA_RecipeBaseConfig>().Where(t => versionIdList.Contains(t.RecipeVersionId) && t.AutoKey == autoKey).ToListAsync();
                var attrConfigList = await _db.Queryable<BPA_RecipeConfig>().Where(t => versionIdList.Contains(t.RecipeVersionId) && t.AutoKey == autoKey).ToListAsync();
                var detailList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => (baseConfigList.Select(a => a.Id).Contains(t.ConfigId) || attrConfigList.Select(a => a.Id).Contains(t.ConfigId)) && t.AutoKey == autoKey).ToListAsync();
                var recipeIds = string.Join(",", recipeList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                string sql = $"DELETE FROM BPA_RecipeInfo WHERE (Id, AutoKey) IN ({recipeIds});";
                var versionIds = string.Join(",", versionList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(versionIds))
                    sql += $"DELETE FROM BPA_RecipeVersion WHERE (Id, AutoKey) IN ({versionIds});";
                var attrIds = string.Join(",", attrList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(attrIds))
                    sql += $"DELETE FROM BPA_RecipeAttribute WHERE (Id, AutoKey) IN ({attrIds});";
                var valueIds = string.Join(",", valueList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(valueIds))
                    sql += $"DELETE FROM BPA_RecipeAttributeValue WHERE (Id, AutoKey) IN ({valueIds});";
                var baseConfigIds = string.Join(",", baseConfigList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(baseConfigIds))
                    sql += $"DELETE FROM BPA_RecipeBaseConfig WHERE (Id, AutoKey) IN ({baseConfigIds});";
                var attrConfigIds = string.Join(",", attrConfigList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(attrConfigIds))
                    sql += $"DELETE FROM BPA_RecipeConfig WHERE (Id, AutoKey) IN ({attrConfigIds});";
                var detailIds = string.Join(",", detailList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(detailIds))
                    sql += $"DELETE FROM BPA_RecipeConfigDetail WHERE (Id, AutoKey) IN ({detailIds});";
                await _db.Ado.ExecuteCommandAsync(sql);
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        public async Task<List<RecipeVersionResDto>> GetRecipeVersionList(string recipeId, int autoKey)
        {
            return await _db.Queryable<BPA_RecipeVersion>().Where(t => t.RecipeId == recipeId && t.AutoKey == autoKey).OrderBy(t => t.Version)
                .Select(t => new RecipeVersionResDto
                {
                    Id = t.Id,
                    RecipeId = t.RecipeId,
                    Version = t.Version,
                    Describe = t.Describe,
                    CreateAt = t.CreateAt
                }).ToListAsync();
        }

        public async Task<bool> UpdateRecipeVersion(RecipeVersionResUpDto dto)
        {
            var versionList = await _db.Queryable<BPA_RecipeVersion>().Where(t => t.RecipeId == dto.RecipeId && t.AutoKey == dto.AutoKey).ToListAsync();
            if (string.IsNullOrWhiteSpace(dto.Id))
            {
                var same = versionList.Any(t => t.Version == dto.Version && t.AutoKey == dto.AutoKey);
                if (same)
                    throw Oops.Oh($"{dto.Version}.0已存在");
                var version = new BPA_RecipeVersion
                {
                    RecipeId = dto.RecipeId,
                    AutoKey = dto.AutoKey,
                    Version = dto.Version,
                    Describe = dto.Describe
                };
                await _db.Insertable(version).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
            }
            else
            {
                var version = versionList.FirstOrDefault(t => t.Id == dto.Id && t.AutoKey == dto.AutoKey);
                if (version == null)
                    throw Oops.Oh($"{dto.Version}.0不存在或已删除");
                if (version.Version != dto.Version)
                {
                    var same = versionList.Any(t => t.Version == dto.Version && t.AutoKey == dto.AutoKey);
                    if (same)
                        throw Oops.Oh($"{dto.Version}.0已存在");
                }
                version.Version = dto.Version;
                version.Describe = dto.Describe;
                await _db.Updateable(version).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
            }
            return true;
        }

        public async Task<bool> DeleteRecipeVersion(string ids, int autoKey)
        {
            try
            {
                _db.Ado.BeginTran();
                var idList = ids.Split(',');
                var isUse = await _db.Queryable<BPA_RecipeTemplate>().AnyAsync(t => idList.Contains(t.VersionId) && t.AutoKey == autoKey);
                if (isUse)
                    throw Oops.Oh("所选配方已被模版使用");
                var versionList = await _db.Queryable<BPA_RecipeVersion>().Where(t => idList.Contains(t.Id) && t.AutoKey == autoKey).ToListAsync();
                var attrList = await _db.Queryable<BPA_RecipeAttribute>().Where(t => idList.Contains(t.RecipeVersionId) && t.AutoKey == autoKey).ToListAsync();
                var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => attrList.Select(a => a.Id).Contains(t.AttributeId) && t.AutoKey == autoKey).ToListAsync();
                var baseConfigList = await _db.Queryable<BPA_RecipeBaseConfig>().Where(t => idList.Contains(t.RecipeVersionId) && t.AutoKey == autoKey).ToListAsync();
                var attrConfigList = await _db.Queryable<BPA_RecipeConfig>().Where(t => idList.Contains(t.RecipeVersionId) && t.AutoKey == autoKey).ToListAsync();
                var detailList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => (baseConfigList.Select(a => a.Id).Contains(t.ConfigId) || attrConfigList.Select(a => a.Id).Contains(t.ConfigId)) && t.AutoKey == autoKey).ToListAsync();
                var versionIds = string.Join(",", versionList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                string sql = $"DELETE FROM BPA_RecipeVersion WHERE (Id, AutoKey) IN ({versionIds});";
                var attrIds = string.Join(",", attrList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(attrIds))
                    sql += $"DELETE FROM BPA_RecipeAttribute WHERE (Id, AutoKey) IN ({attrIds});";
                var valueIds = string.Join(",", valueList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(valueIds))
                    sql += $"DELETE FROM BPA_RecipeAttributeValue WHERE (Id, AutoKey) IN ({valueIds});";
                var baseConfigIds = string.Join(",", baseConfigList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(baseConfigIds))
                    sql += $"DELETE FROM BPA_RecipeBaseConfig WHERE (Id, AutoKey) IN ({baseConfigIds});";
                var attrConfigIds = string.Join(",", attrConfigList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(attrConfigIds))
                    sql += $"DELETE FROM BPA_RecipeConfig WHERE (Id, AutoKey) IN ({attrConfigIds});";
                var detailIds = string.Join(",", detailList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(detailIds))
                    sql += $"DELETE FROM BPA_RecipeConfigDetail WHERE (Id, AutoKey) IN ({detailIds});";
                await _db.Ado.ExecuteCommandAsync(sql);
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 获取基本配方详情
        /// </summary>
        /// <param name="recipeVersionId"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<List<RecipeBatchingDto>> GetBaseBatchingList(string recipeVersionId, int autoKey)
        {
            var config = _db.Queryable<BPA_RecipeBaseConfig>().First(t => t.RecipeVersionId == recipeVersionId && t.AutoKey == autoKey);
            if (config == null)
                return await GetNullBatchingList();
            var batchingList = await _db.Queryable<BPA_Batching>().ToListAsync();
            var detailList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => t.ConfigId == config.Id && t.AutoKey == autoKey).ToListAsync();
            var recipeBatchingList = new List<RecipeBatchingDto>();
            foreach (var batching in batchingList)
            {
                recipeBatchingList.Add(new RecipeBatchingDto
                {
                    Id = Guid.NewGuid().ToString(),
                    BatchingId = batching.Id,
                    Name = batching.Name,
                    Weight = detailList.FirstOrDefault(t => t.ConfigId == config.Id && t.BatchingId == batching.Id && t.AutoKey == autoKey)?.Weight ?? 0
                });
            }
            return recipeBatchingList.OrderByDescending(t => t.Weight).ToList();
        }

        /// <summary>
        /// 获取多属性配方详情
        /// </summary>
        /// <param name="recipeVersionId"></param>
        /// <param name="attrIds"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<List<RecipeBatchingDto>> GetAttrBatchingList(string recipeVersionId, string attrIds, int autoKey)
        {
            var batchingList = await _db.Queryable<BPA_Batching>().ToListAsync();
            var configList = await _db.Queryable<BPA_RecipeConfig>().Where(t => t.RecipeVersionId == recipeVersionId && t.AutoKey == autoKey).ToListAsync();
            var detailList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => configList.Select(a => a.Id).Contains(t.ConfigId) && t.AutoKey == autoKey).ToListAsync();
            var recipeBatchingList = new List<RecipeBatchingDto>();
            foreach (var config in configList)
            {
                if (CompareAttrId(config.AttributeId, attrIds))
                {
                    foreach (var batching in batchingList)
                    {
                        recipeBatchingList.Add(new RecipeBatchingDto
                        {
                            Id = Guid.NewGuid().ToString(),
                            BatchingId = batching.Id,
                            Name = batching.Name,
                            Weight = detailList.FirstOrDefault(t => t.ConfigId == config.Id && t.BatchingId == batching.Id && t.AutoKey == autoKey)?.Weight ?? 0
                        });
                    }
                    return recipeBatchingList.OrderByDescending(t => t.Weight).ToList();
                }
            }
            return await GetNullBatchingList();
        }

        public static bool CompareAttrId(string attributeIds, string attrIds)
        {
            var attributeIdSet = new HashSet<string>(attributeIds.Split(',').ToList());
            var attrIdSet = new HashSet<string>(attrIds.Split(',').ToList());
            return attributeIdSet.SetEquals(attrIdSet);
        }

        public async Task<List<RecipeBatchingDto>> GetNullBatchingList()
        {
            return await _db.Queryable<BPA_Batching>().OrderBy(t => t.Name).Select(a => new RecipeBatchingDto
            {
                BatchingId = a.Id,
                Name = a.Name,
                Weight = 0
            }).Mapper(t =>
            {
                t.Id = Guid.NewGuid().ToString();
            }).ToListAsync();
        }

        /// <summary>
        /// 设置配方物料
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> SetRecipeBatching(RecipeBatchingDataDto dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var baseAddList = new List<BPA_RecipeBaseConfig>();
                var baseUpdateList = new List<BPA_RecipeBaseConfig>();
                var attrAddList = new List<BPA_RecipeConfig>();
                var attrUpdateList = new List<BPA_RecipeConfig>();
                var baseDetailAddList = new List<BPA_RecipeConfigDetail>();
                var baseDetailUpdateList = new List<BPA_RecipeConfigDetail>();
                var configDetailList = await _db.Queryable<BPA_RecipeConfigDetail>().ToListAsync();
                if (string.IsNullOrWhiteSpace(dto.ValueIds))
                {
                    var baseConfig = await _db.Queryable<BPA_RecipeBaseConfig>().FirstAsync(t => t.RecipeVersionId == dto.RecipeVersionId && t.AutoKey == dto.AutoKey);
                    if (baseConfig != null)
                    {
                        baseUpdateList.Add(baseConfig);
                        var detailList = configDetailList.Where(t => t.ConfigId == baseConfig.Id && t.AutoKey == dto.AutoKey).ToList();
                        foreach (var batching in dto.BatchingList)
                        {
                            var detail = detailList.FirstOrDefault(t => t.BatchingId == batching.BatchingId && t.AutoKey == dto.AutoKey);
                            if (detail != null)
                            {
                                detail.Weight = batching.Weight;
                                baseDetailUpdateList.Add(detail);
                            }
                            else
                            {
                                baseDetailAddList.Add(new BPA_RecipeConfigDetail
                                {
                                    ConfigId = baseConfig.Id,
                                    BatchingId = batching.BatchingId,
                                    Weight = batching.Weight,
                                    AutoKey = dto.AutoKey
                                });
                            }
                        }
                    }
                    else
                    {
                        var id = Guid.NewGuid().ToString();
                        baseAddList.Add(new BPA_RecipeBaseConfig
                        {
                            Id = id,
                            RecipeVersionId = dto.RecipeVersionId,
                            AutoKey = dto.AutoKey
                        });
                        foreach (var batching in dto.BatchingList)
                        {
                            baseDetailAddList.Add(new BPA_RecipeConfigDetail
                            {
                                ConfigId = id,
                                BatchingId = batching.BatchingId,
                                Weight = batching.Weight,
                                AutoKey = dto.AutoKey
                            });
                        }
                    }
                    if (baseAddList.Count > 0)
                        await _db.Insertable(baseAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    if (baseDetailAddList.Count > 0)
                        await _db.Insertable(baseDetailAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    await _db.Updateable(baseDetailUpdateList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                    await _db.Updateable(baseUpdateList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                }
                else
                {
                    var configList = await _db.Queryable<BPA_RecipeConfig>().Where(t => t.RecipeVersionId == dto.RecipeVersionId && t.AutoKey == dto.AutoKey).ToListAsync();
                    var haveConfig = false;
                    foreach (var config in configList)
                    {
                        if (CompareAttrId(config.AttributeId, dto.ValueIds))
                        {
                            attrUpdateList.Add(config);
                            var detailList = configDetailList.Where(t => t.ConfigId == config.Id && t.AutoKey == dto.AutoKey).ToList();
                            foreach (var batching in dto.BatchingList)
                            {
                                var detail = detailList.FirstOrDefault(t => t.BatchingId == batching.BatchingId && t.AutoKey == dto.AutoKey);
                                if (detail != null)
                                {
                                    detail.Weight = batching.Weight;
                                    baseDetailUpdateList.Add(detail);
                                }
                                else
                                {
                                    baseDetailAddList.Add(new BPA_RecipeConfigDetail
                                    {
                                        ConfigId = config.Id,
                                        BatchingId = batching.BatchingId,
                                        Weight = batching.Weight,
                                        AutoKey = dto.AutoKey
                                    });
                                }
                            }
                            if (baseDetailAddList.Count > 0)
                                await _db.Insertable(baseDetailAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                            await _db.Updateable(baseDetailUpdateList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                            await _db.Updateable(attrUpdateList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                            haveConfig = true;
                            break;
                        }
                    }
                    if (!haveConfig)
                    {
                        var id = Guid.NewGuid().ToString();
                        attrAddList.Add(new BPA_RecipeConfig
                        {
                            Id = id,
                            RecipeVersionId = dto.RecipeVersionId,
                            AttributeId = dto.ValueIds,
                            AutoKey = dto.AutoKey
                        });
                        foreach (var batching in dto.BatchingList)
                        {
                            baseDetailAddList.Add(new BPA_RecipeConfigDetail
                            {
                                ConfigId = id,
                                BatchingId = batching.BatchingId,
                                Weight = batching.Weight,
                                AutoKey = dto.AutoKey
                            });
                        }
                        if (attrAddList.Count > 0)
                            await _db.Insertable(attrAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                        if (baseDetailAddList.Count > 0)
                            await _db.Insertable(baseDetailAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    }
                }
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 获取已配置多属性配方列表
        /// </summary>
        /// <param name="recipeVersionId"></param>
        /// <param name="valueIds"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<List<AttrRecipeListDto>> GetAttrRecipeList(string recipeVersionId, string valueIds, int autoKey)
        {
            var configList = await _db.Queryable<BPA_RecipeConfig>().Where(t => t.RecipeVersionId == recipeVersionId && t.AutoKey == autoKey).OrderBy(t => t.AttributeId).ToListAsync();
            var attrList = await _db.Queryable<BPA_RecipeAttribute>().Where(t => t.RecipeVersionId == recipeVersionId && t.AutoKey == autoKey).ToListAsync();
            var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => attrList.Select(a => a.Id).Contains(t.AttributeId) && t.AutoKey == autoKey).ToListAsync();
            var result = new List<AttrRecipeListDto>();
            if (valueIds != null)
            {
                var valueIdList = valueIds?.Split(',');
                foreach (var valueId in valueIdList)
                {
                    configList = configList.Where(t => t.AttributeId.Contains(valueId) && t.AutoKey == autoKey).ToList();
                }
            }
            foreach (var config in configList)
            {
                var names = "";
                var idList = config.AttributeId.Split(',');
                foreach (var attrId in idList)
                {
                    var name = valueList.FirstOrDefault(t => t.Id == attrId && t.AutoKey == autoKey)?.Name;
                    names = names + "-" + name;
                }
                result.Add(new AttrRecipeListDto
                {
                    Id = config.Id,
                    ValueIds = config.AttributeId,
                    Name = names.TrimStart('-'),
                    BatchingList = await GetAttrBatchingList(recipeVersionId, config.AttributeId, autoKey)
                });
            }
            return result;
        }

        /// <summary>
        /// 删除多属性物料列表
        /// </summary>
        /// <param name="configId"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<bool> DeleteRecipeAttr(string configId, int autoKey)
        {
            try
            {
                _db.Ado.BeginTran();
                await _db.Deleteable<BPA_RecipeConfig>().Where(t => t.Id == configId && t.AutoKey == autoKey).ExecuteCommandAsync();
                await _db.Deleteable<BPA_RecipeConfigDetail>().Where(t => t.ConfigId == configId && t.AutoKey == autoKey).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        #endregion

        #region 配方模版
        /// <summary>
        /// 获取配方分组
        /// </summary>
        /// <param name="groupTempId"></param>
        /// <param name="isWeight"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<List<RecipeGroupDto>> GetRecipeGroupList(string groupTempId, bool isWeight, int autoKey)
        {
            return await _db.Queryable<BPA_RecipeGroup>()
                .WhereIF(!string.IsNullOrWhiteSpace(groupTempId), x => x.GroupTempId == groupTempId)
                .Where(t => t.IsWeight == isWeight && t.AutoKey == autoKey)
                .OrderBy(t => t.Sort).Select(t => new RecipeGroupDto
                {
                    Id = t.Id,
                    Name = t.Name,
                    IsWeight = isWeight,
                    Sort = t.Sort
                }).ToListAsync();
        }

        /// <summary>
        /// 更新配方分组
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdataRecipeGroup(RecipeGroupDto dto)
        {
            try
            {
                var res = 0;
                var groupList = await _db.Queryable<BPA_RecipeGroup>().WhereIF(!string.IsNullOrWhiteSpace(dto.GroupTempId), x => x.GroupTempId == dto.GroupTempId).ToListAsync();
                if (dto.Id == null)
                {
                    var same = groupList.Any(t => t.Name == dto.Name && t.IsWeight == dto.IsWeight && t.AutoKey == dto.AutoKey);
                    if (same)
                        throw Oops.Oh($"{dto.Name}已存在");
                    var id = Guid.NewGuid().ToString();
                    var group = new BPA_RecipeGroup
                    {
                        Id = id,
                        GroupTempId = dto.GroupTempId,
                        Name = dto.Name,
                        IsWeight = dto.IsWeight,
                        Sort = dto.Sort,
                        AutoKey = dto.AutoKey
                    };
                    res = await _db.Insertable(group).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    await AddRecipeTemplatePage(new RecipePageDto { RecipeGroupId = id, AutoKey = dto.AutoKey });
                }
                else
                {
                    var group = groupList.FirstOrDefault(t => t.Id == dto.Id && t.AutoKey == dto.AutoKey);
                    if (group == null)
                        throw Oops.Oh($"{dto.Name}不存在或已删除");
                    if (group.Name == dto.Name)
                    {
                        if (group.IsWeight == dto.IsWeight)
                            group.Sort = dto.Sort;
                        else
                        {
                            var same = groupList.Any(t => t.Name == dto.Name && t.IsWeight == dto.IsWeight && t.AutoKey == dto.AutoKey);
                            if (same)
                                throw Oops.Oh($"{dto.Name}已存在");
                        }
                    }
                    else
                    {
                        var same = groupList.Any(t => t.Name == dto.Name && t.IsWeight == dto.IsWeight && t.AutoKey == dto.AutoKey);
                        if (same)
                            throw Oops.Oh($"{dto.Name}已存在");
                        group.Name = dto.Name;
                        group.IsWeight = dto.IsWeight;
                        group.Sort = dto.Sort;
                    }
                    res = await _db.Updateable(group).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                }
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 增加一页配方
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddRecipeTemplatePage(RecipePageDto dto)
        {
            try
            {
                var templateAddList = new List<BPA_RecipeTemplate>();
                var maxSort = await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.RecipeGroupId == dto.RecipeGroupId && t.AutoKey == dto.AutoKey).MaxAsync(a => a.Sort);
                for (int i = 1; i <= 20; i++)
                {
                    templateAddList.Add(new BPA_RecipeTemplate
                    {
                        RecipeGroupId = dto.RecipeGroupId,
                        Sort = i + maxSort,
                        AutoKey = dto.AutoKey
                    });
                }
                if (templateAddList.Count > 0)
                    await _db.Insertable(templateAddList).CallEntityMethod(t => t.Create()).ExecuteCommandAsync();
                return true;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 删除当前页所有配方
        /// </summary>
        /// <param name="recipeGroupId"></param>
        /// <param name="page"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<bool> DeleteRecipeTemplatePage(string recipeGroupId, int page, int autoKey)
        {
            var skipCount = (page - 1) * 20;
            var templateDelList = await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.RecipeGroupId == recipeGroupId && t.AutoKey == autoKey).OrderBy(a => a.Sort).Skip(skipCount).Take(20).ToListAsync();
            if (templateDelList.Any())
            {
                var ids = string.Join(",", templateDelList.Select(t => $"('{t.Id}', {t.AutoKey})"));
                if (!string.IsNullOrWhiteSpace(ids))
                {
                    string sql = $"DELETE FROM BPA_RecipeTemplate WHERE (Id, AutoKey) IN ({ids})";
                    await _db.Ado.ExecuteCommandAsync(sql);
                }
            }
            var ishave = await _db.Queryable<BPA_RecipeTemplate>().AnyAsync(t => t.RecipeGroupId == recipeGroupId && t.AutoKey == autoKey);
            if (!ishave)
                await AddRecipeTemplatePage(new RecipePageDto { RecipeGroupId = recipeGroupId, AutoKey = autoKey });
            return true;
        }

        /// <summary>
        /// 删除配方分组
        /// </summary>
        /// <param name="id"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<bool> DeleteRecipeGroup(string id, int autoKey)
        {
            try
            {
                _db.Ado.BeginTran();
                var group = await _db.Queryable<BPA_RecipeGroup>().FirstAsync(t => t.Id == id && t.AutoKey == autoKey);
                var templateList = await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.RecipeGroupId == id && t.AutoKey == autoKey).ToListAsync();
                var isHave = templateList.Any(t => !string.IsNullOrWhiteSpace(t.RecipeId) && t.AutoKey == autoKey);
                if (isHave)
                    throw Oops.Oh("该分组已绑定配方，请先删除绑定的配方");
                await _db.Deleteable<BPA_RecipeGroup>().Where(t => t.Id == id && t.AutoKey == autoKey).ExecuteCommandAsync();
                if (templateList.Any())
                {
                    var ids = string.Join(",", templateList.Select(t => $"('{t.Id}', {t.AutoKey})"));
                    if (!string.IsNullOrWhiteSpace(ids))
                    {
                        string sql = $"DELETE FROM BPA_RecipeTemplate WHERE (Id, AutoKey) IN ({ids})";
                        await _db.Ado.ExecuteCommandAsync(sql);
                    }
                }
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 通过配方分组获取配方数量
        /// </summary>
        /// <param name="recipeGroupId"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<int> GetRecipeTemplateCount(string recipeGroupId, int autoKey)
        {
            return await _db.Queryable<BPA_RecipeTemplate>().CountAsync(t => t.RecipeGroupId == recipeGroupId && t.AutoKey == autoKey);
        }

        /// <summary>
        /// 通过配方分组和页数获取配方信息
        /// </summary>
        /// <param name="recipeGroupId"></param>
        /// <param name="page"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<List<RecipeTemplateInfoDto>> GetRecipeTemplateList(string recipeGroupId, int page, int autoKey)
        {
            var skipCount = (page - 1) * 20;
            var recipeList = await _db.Queryable<BPA_RecipeInfo>().ToListAsync();
            var versionList = await _db.Queryable<BPA_RecipeVersion>().ToListAsync();
            return await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.RecipeGroupId == recipeGroupId && t.AutoKey == autoKey)
                .OrderBy(a => a.Sort).Skip(skipCount).Take(20).Select(t => new RecipeTemplateInfoDto
                {
                    Id = t.Id,
                    RecipeGroupId = t.RecipeGroupId,
                    RecipeId = t.RecipeId,
                    RecipeName = "",
                    VersionId = t.VersionId,
                    Version = null,
                    Describe = null,
                    Sort = t.Sort
                }).Mapper(a =>
                {
                    var recipe = recipeList.FirstOrDefault(t => t.Id == a.RecipeId && t.AutoKey == autoKey);
                    if (recipe != null)
                        a.RecipeName = recipe.Name;
                    var version = versionList.FirstOrDefault(t => t.Id == a.VersionId && t.AutoKey == autoKey);
                    if (version != null)
                    {
                        a.Version = version.Version;
                        a.Describe = version.Describe;
                    }
                }).ToListAsync();
        }

        public async Task<bool> UpdateRecipeTemplate(RecipeTemplateUpDto dto)
        {
            var templateList = await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.AutoKey == dto.AutoKey).ToListAsync();
            var template = templateList.FirstOrDefault(t => t.Id == dto.Id);
            if (template == null)
                throw Oops.Oh("数据不存在或已删除");
            template.RecipeId = null;
            template.VersionId = null;
            await _db.Updateable(template).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
            var recipe = await _db.Queryable<BPA_RecipeInfo>().FirstAsync(t => t.Id == dto.RecipeId && t.AutoKey == dto.AutoKey);
            var same = templateList.Any(t => t.RecipeId == dto.RecipeId);
            if (same)
                throw Oops.Oh($"模版已存在{recipe.Name}");
            template.RecipeId = dto.RecipeId;
            template.VersionId = dto.VersionId;
            await _db.Updateable(template).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
            return true;
        }

        public async Task<bool> DeleteRecipeTemplate(string id, int autoKey)
        {
            var template = await _db.Queryable<BPA_RecipeTemplate>().FirstAsync(t => t.Id == id && t.AutoKey == autoKey);
            if (template == null)
                throw Oops.Oh("数据不存在或已删除");
            template.RecipeId = null;
            template.VersionId = null;
            await _db.Updateable(template).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
            return true;
        }

        public async Task<List<RecipeSelectDto>> GetRecipeVersionSelectList(string recipeId, bool isWeight, int autoKey)
        {
            var versionList = await _db.Queryable<BPA_RecipeVersion>().ToListAsync();
            var templateRecipeList = await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.AutoKey == autoKey && !string.IsNullOrWhiteSpace(t.RecipeId) && t.RecipeId != recipeId).Select(a => a.RecipeId).ToListAsync();
            var res = await _db.Queryable<BPA_RecipeInfo>()
                .Where(t => !templateRecipeList.Contains(t.Id))
                .Where(t => t.IsWeight == isWeight && t.AutoKey == autoKey)
                .OrderByDescending(a => a.CreateAt)
                .Select(t => new RecipeSelectDto
                {
                    RecipeId = t.Id,
                    Name = t.Name,
                    VersionList = new List<VersionSelectDto>()
                }).Mapper(a =>
                {
                    a.VersionList = versionList.Where(t => t.RecipeId == a.RecipeId && t.AutoKey == autoKey).OrderBy(a => a.Version).Select(t => new VersionSelectDto
                    {
                        VersionId = t.Id,
                        Version = t.Version + ".0",
                        Describe = t.Describe
                    }).ToList();
                }).ToListAsync();
            return res;
        }

        public async Task<int> GetMaxVersion(string recipeId, int autoKey)
        {
            return await _db.Queryable<BPA_RecipeVersion>().Where(t => t.RecipeId == recipeId && t.AutoKey == autoKey).MaxAsync(a => a.Version);
        }

        public async Task<int> GetGroupMaxSort(bool isWeight, int autoKey, string groupTempId)
        {
            return await _db.Queryable<BPA_RecipeGroup>()
                .WhereIF(!string.IsNullOrWhiteSpace(groupTempId), x => x.GroupTempId == groupTempId)
                .Where(t => t.IsWeight == isWeight && t.AutoKey == autoKey).MaxAsync(a => a.Sort);
        }

        public async Task<List<CopyItemDto>> GetCopyList(int autoKey)
        {
            var result = new List<CopyItemDto>();
            var groupId = App.User?.FindFirst(ClaimConst.GroupId)?.Value;
            var company = await _db.Queryable<BPA_Company>().FirstAsync(t => t.Id == groupId);
            if (autoKey != 0)
                result.Add(new CopyItemDto
                {
                    Value = 0,
                    Label = $"基础配方"
                });
            result.AddRange(await _db.Queryable<BPA_DeviceInfo>().Where(t => t.AutoKey != autoKey).OrderBy(t => t.AutoKey).Select(a => new CopyItemDto
            {
                Value = a.AutoKey,
                Label = a.Name
            }).ToListAsync());
            return result;
        }

        public async Task<List<RecipeResDto>> GetAllRecipeList(int autoKey)
        {
            return await _db.Queryable<BPA_RecipeInfo>().Where(t => t.AutoKey == autoKey).OrderByDescending(t => t.CreateAt).Select(a => new RecipeResDto
            {
                Id = a.Id,
                Name = a.Name,
                IsWeight = a.IsWeight,
                ReferenceWeight = a.ReferenceWeight,
                CreateAt = a.CreateAt
            }).ToListAsync();
        }

        public async Task<List<RecipeGroupTemplateDto>> GetTemplateList(bool select, int autoKey, string groupTempId)
        {
            if(select)
            {
                return await _db.Queryable<BPA_DeviceInfo>().Where(t => t.AutoKey != autoKey).OrderBy(t => t.AutoKey).Select(a => new RecipeGroupTemplateDto
                {
                    Id = a.AutoKey.ToString(),
                    Name = a.Name
                }).ToListAsync();
            }
            else
            {
                return await _db.Queryable<BPA_RecipeGroupTemplate>().Where(t => t.Id != groupTempId).OrderBy(t => t.CreateAt).Select(t => new RecipeGroupTemplateDto
                {
                    Id = t.Id,
                    Name = t.Name
                }).ToListAsync();
            }
        }

        public async Task<List<RecipeResDto>> GetRecipeListWithName(string name, int autoKey)
        {
            return await _db.Queryable<BPA_RecipeInfo>()
                .WhereIF(!string.IsNullOrWhiteSpace(name), x => x.Name.Contains(name))
                .Where(t => t.AutoKey == autoKey)
                .OrderByDescending(t => t.CreateAt)
                .Select(a => new RecipeResDto
                {
                    Id = a.Id,
                    Name = a.Name,
                }).ToListAsync();
        }

        public async Task<List<RecipeResDto>> GetRepeatRecipeList(string names, int selectAutoKey, int autoKey)
        {
            if (!string.IsNullOrWhiteSpace(names))
            {
                var nameList = names.Split(',');
                return await _db.Queryable<BPA_RecipeInfo>().Where(t => nameList.Contains(t.Name) && t.AutoKey == autoKey).OrderByDescending(t => t.CreateAt).Select(a => new RecipeResDto
                {
                    Id = a.Id,
                    Name = a.Name,
                    IsWeight = a.IsWeight,
                    ReferenceWeight = a.ReferenceWeight,
                    CreateAt = a.CreateAt
                }).ToListAsync();
            }
            else
            {
                var nameList = await _db.Queryable<BPA_RecipeInfo>().Where(t => t.AutoKey == selectAutoKey).Select(a => a.Name).ToListAsync();
                return await _db.Queryable<BPA_RecipeInfo>().Where(t => nameList.Contains(t.Name) && t.AutoKey == autoKey).OrderByDescending(t => t.CreateAt).Select(a => new RecipeResDto
                {
                    Id = a.Id,
                    Name = a.Name,
                    IsWeight = a.IsWeight,
                    ReferenceWeight = a.ReferenceWeight,
                    CreateAt = a.CreateAt
                }).ToListAsync();
            }
        }

        public async Task<bool> CopyRecipeList(string names, string skipNames, int newAutoKey, int oldAutoKey)
        {
            try
            {
                _db.Ado.BeginTran();
                string[] allNameList;
                if (!string.IsNullOrWhiteSpace(names))
                    allNameList = names.Split(',');
                else
                    allNameList = await _db.Queryable<BPA_RecipeInfo>().Where(t => t.AutoKey == newAutoKey).Select(a => a.Name).ToArrayAsync();
                string[] nameList;
                if (skipNames != null)
                {
                    var skipNameList = skipNames?.Split(',');
                    nameList = allNameList.Where(t => !skipNameList.Contains(t)).ToArray();
                }
                else
                    nameList = allNameList;
                var newRecipeList = await _db.Queryable<BPA_RecipeInfo>().Where(t => nameList.Contains(t.Name) && t.AutoKey == newAutoKey).ToListAsync();
                var oldRecipeList = await _db.Queryable<BPA_RecipeInfo>().Where(t => t.AutoKey == oldAutoKey).ToListAsync();
                var templateList = await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.AutoKey == oldAutoKey).ToListAsync();
                var versionList = await _db.Queryable<BPA_RecipeVersion>().Where(t => newRecipeList.Select(t => t.Id).Contains(t.RecipeId) && t.AutoKey == newAutoKey).ToListAsync();
                var versionIdList = versionList.Select(t => t.Id).ToList();
                var attrList = await _db.Queryable<BPA_RecipeAttribute>().Where(t => versionIdList.Contains(t.RecipeVersionId) && t.AutoKey == newAutoKey).ToListAsync();
                var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => attrList.Select(a => a.Id).Contains(t.AttributeId) && t.AutoKey == newAutoKey).ToListAsync();
                var baseConfigList = await _db.Queryable<BPA_RecipeBaseConfig>().Where(t => versionIdList.Contains(t.RecipeVersionId) && t.AutoKey == newAutoKey).ToListAsync();
                var attrConfigList = await _db.Queryable<BPA_RecipeConfig>().Where(t => versionIdList.Contains(t.RecipeVersionId) && t.AutoKey == newAutoKey).ToListAsync();
                var detailList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => (baseConfigList.Select(a => a.Id).Contains(t.ConfigId) || attrConfigList.Select(a => a.Id).Contains(t.ConfigId)) && t.AutoKey == newAutoKey).ToListAsync();
                var recipeAddList = new List<BPA_RecipeInfo>();
                var versionAddList = new List<BPA_RecipeVersion>();
                var attrAddList = new List<BPA_RecipeAttribute>();
                var valueAddList = new List<BPA_RecipeAttributeValue>();
                var baseConfigAddList = new List<BPA_RecipeBaseConfig>();
                var attrConfigAddList = new List<BPA_RecipeConfig>();
                var detailAddList = new List<BPA_RecipeConfigDetail>();
                var recipeDelList = new List<BPA_RecipeInfo>();
                foreach (var newRecipe in newRecipeList)
                {
                    var oldRecipe = oldRecipeList.FirstOrDefault(t => t.Name == newRecipe.Name);
                    if (oldRecipe != null)
                    {
                        var isUse = templateList.Any(t => t.RecipeId == oldRecipe.Id);
                        if (isUse)
                            throw Oops.Oh($"导入失败：{oldRecipe.Name}已被模版使用");
                        recipeDelList.Add(oldRecipe);
                    }
                    recipeAddList.Add(new BPA_RecipeInfo
                    {
                        Id = newRecipe.Id,
                        AutoKey = oldAutoKey,
                        Name = newRecipe.Name,
                        IsWeight = newRecipe.IsWeight,
                        ReferenceWeight = newRecipe.ReferenceWeight
                    });
                }
                versionAddList.AddRange(versionList.Select(version => new BPA_RecipeVersion
                {
                    Id = version.Id,
                    AutoKey = oldAutoKey,
                    RecipeId = version.RecipeId,
                    Version = version.Version,
                    Describe = version.Describe
                }));
                attrAddList.AddRange(attrList.Select(attr => new BPA_RecipeAttribute
                {
                    Id = attr.Id,
                    AutoKey = oldAutoKey,
                    RecipeVersionId = attr.RecipeVersionId,
                    Name = attr.Name,
                    Sort = attr.Sort
                }));
                valueAddList.AddRange(valueList.Select(value => new BPA_RecipeAttributeValue
                {
                    Id = value.Id,
                    AutoKey = oldAutoKey,
                    AttributeId = value.AttributeId,
                    Name = value.Name,
                    Sort = value.Sort
                }));
                baseConfigAddList.AddRange(baseConfigList.Select(baseConfig => new BPA_RecipeBaseConfig
                {
                    Id = baseConfig.Id,
                    AutoKey = oldAutoKey,
                    RecipeVersionId = baseConfig.RecipeVersionId
                }));
                attrConfigAddList.AddRange(attrConfigList.Select(attrConfig => new BPA_RecipeConfig
                {
                    Id = attrConfig.Id,
                    AutoKey = oldAutoKey,
                    RecipeVersionId = attrConfig.RecipeVersionId,
                    AttributeId = attrConfig.AttributeId
                }));
                detailAddList.AddRange(detailList.Select(detail => new BPA_RecipeConfigDetail
                {
                    Id = detail.Id,
                    AutoKey = oldAutoKey,
                    ConfigId = detail.ConfigId,
                    BatchingId = detail.BatchingId,
                    Weight = detail.Weight
                }));
                if (recipeDelList.Count > 0)
                {
                    var versionDelList = await _db.Queryable<BPA_RecipeVersion>().Where(t => recipeDelList.Select(a => a.Id).Contains(t.RecipeId) && t.AutoKey == oldAutoKey).ToListAsync();
                    var versionDelIdList = versionDelList.Select(t => t.Id);
                    var attrDelList = await _db.Queryable<BPA_RecipeAttribute>().Where(t => versionDelIdList.Contains(t.RecipeVersionId) && t.AutoKey == oldAutoKey).ToListAsync();
                    var valueDelList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => attrDelList.Select(a => a.Id).Contains(t.AttributeId) && t.AutoKey == oldAutoKey).ToListAsync();
                    var baseConfigDelList = await _db.Queryable<BPA_RecipeBaseConfig>().Where(t => versionDelIdList.Contains(t.RecipeVersionId) && t.AutoKey == oldAutoKey).ToListAsync();
                    var attrConfigDelList = await _db.Queryable<BPA_RecipeConfig>().Where(t => versionDelIdList.Contains(t.RecipeVersionId) && t.AutoKey == oldAutoKey).ToListAsync();
                    var detailDelList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => (baseConfigDelList.Select(a => a.Id).Contains(t.ConfigId) || attrConfigDelList.Select(a => a.Id).Contains(t.ConfigId)) && t.AutoKey == oldAutoKey).ToListAsync();
                    var recipeIds = string.Join(",", recipeDelList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                    string sql = $"DELETE FROM BPA_RecipeInfo WHERE (Id, AutoKey) IN ({recipeIds});";
                    var versionIds = string.Join(",", versionDelList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                    if (!string.IsNullOrWhiteSpace(versionIds))
                        sql += $"DELETE FROM BPA_RecipeVersion WHERE (Id, AutoKey) IN ({versionIds});";
                    var attrIds = string.Join(",", attrDelList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                    if (!string.IsNullOrWhiteSpace(attrIds))
                        sql += $"DELETE FROM BPA_RecipeAttribute WHERE (Id, AutoKey) IN ({attrIds});";
                    var valueIds = string.Join(",", valueDelList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                    if (!string.IsNullOrWhiteSpace(valueIds))
                        sql += $"DELETE FROM BPA_RecipeAttributeValue WHERE (Id, AutoKey) IN ({valueIds});";
                    var baseConfigIds = string.Join(",", baseConfigDelList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                    if (!string.IsNullOrWhiteSpace(baseConfigIds))
                        sql += $"DELETE FROM BPA_RecipeBaseConfig WHERE (Id, AutoKey) IN ({baseConfigIds});";
                    var attrConfigIds = string.Join(",", attrConfigDelList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                    if (!string.IsNullOrWhiteSpace(attrConfigIds))
                        sql += $"DELETE FROM BPA_RecipeConfig WHERE (Id, AutoKey) IN ({attrConfigIds});";
                    var detailIds = string.Join(",", detailDelList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                    if (!string.IsNullOrWhiteSpace(detailIds))
                        sql += $"DELETE FROM BPA_RecipeConfigDetail WHERE (Id, AutoKey) IN ({detailIds});";
                    await _db.Ado.ExecuteCommandAsync(sql);
                }
                if (recipeAddList.Count > 0)
                    await _db.Insertable(recipeAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (versionAddList.Count > 0)
                    await _db.Insertable(versionAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (attrAddList.Count > 0)
                    await _db.Insertable(attrAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (valueAddList.Count > 0)
                    await _db.Insertable(valueAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (baseConfigAddList.Count > 0)
                    await _db.Insertable(baseConfigAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (attrConfigAddList.Count > 0)
                    await _db.Insertable(attrConfigAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (detailAddList.Count > 0)
                    await _db.Insertable(detailAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        public async Task<bool> CopyTemplateList(string newAutoKey, int oldAutoKey, string groupTempId)
        {
            try
            {
                var oldGroupList = await _db.Queryable<BPA_RecipeGroup>()
                    .WhereIF(!string.IsNullOrWhiteSpace(groupTempId), x => x.GroupTempId == groupTempId)
                    .Where(t => t.AutoKey == oldAutoKey).ToListAsync();
                var oldTemplateList = await _db.Queryable<BPA_RecipeTemplate>()
                    .WhereIF(!string.IsNullOrWhiteSpace(groupTempId), x => oldGroupList.Select(a => a.Id).Contains(x.RecipeGroupId))
                    .Where(t => t.AutoKey == oldAutoKey).ToListAsync();
                if (oldGroupList.Any())
                {
                    var ids = string.Join(",", oldGroupList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                    if (!string.IsNullOrWhiteSpace(ids))
                    {
                        string sql = $"DELETE FROM BPA_RecipeGroup WHERE (Id, AutoKey) IN ({ids});";
                        if (oldTemplateList.Any())
                        {
                            var tIds = string.Join(",", oldTemplateList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                            if (!string.IsNullOrWhiteSpace(tIds))
                                sql += $"DELETE FROM BPA_RecipeTemplate WHERE (Id, AutoKey) IN ({tIds});";
                        }
                        await _db.Ado.ExecuteCommandAsync(sql);
                    }
                }
                _db.Ado.BeginTran();
                var groupAddList = new List<BPA_RecipeGroup>();
                var templateAddList = new List<BPA_RecipeTemplate>();
                var oldRecipeList = await _db.Queryable<BPA_RecipeInfo>().Where(t => t.AutoKey == oldAutoKey).ToListAsync();
                var newGroupList = new List<BPA_RecipeGroup>();
                var newTemplateList = new List<BPA_RecipeTemplate>();
                if (newAutoKey.Contains('-'))
                {
                    newGroupList = await _db.Queryable<BPA_RecipeGroup>().Where(t => t.GroupTempId == newAutoKey).ToListAsync();
                    newTemplateList = await _db.Queryable<BPA_RecipeTemplate>().Where(t => newGroupList.Select(a => a.Id).Contains(t.RecipeGroupId) && t.AutoKey == 0).ToListAsync();
                }
                else
                {
                    newGroupList = await _db.Queryable<BPA_RecipeGroup>().Where(t => t.AutoKey == int.Parse(newAutoKey)).ToListAsync();
                    newTemplateList = await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.AutoKey == int.Parse(newAutoKey)).ToListAsync();
                }
                foreach (var group in newGroupList)
                {
                    var id = Guid.NewGuid().ToString();
                    groupAddList.Add(new BPA_RecipeGroup
                    {
                        Id = id,
                        GroupTempId = !string.IsNullOrWhiteSpace(groupTempId) ? groupTempId : null,
                        Name = group.Name,
                        IsWeight = group.IsWeight,
                        Sort = group.Sort,
                        AutoKey = oldAutoKey
                    });
                    var templateList = newTemplateList.Where(t => t.RecipeGroupId == group.Id).ToList();
                    foreach (var temp in templateList)
                    {
                        var recipeHave = oldRecipeList.Any(t => t.Id == temp.RecipeId);
                        templateAddList.Add(new BPA_RecipeTemplate
                        {
                            Id = Guid.NewGuid().ToString(),
                            RecipeGroupId = id,
                            RecipeId = recipeHave ? temp.RecipeId : null,
                            VersionId = recipeHave ? temp.VersionId : null,
                            Sort = temp.Sort,
                            AutoKey = oldAutoKey
                        });
                    }
                }
                if (groupAddList.Count > 0)
                    await _db.Insertable(groupAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (templateAddList.Count > 0)
                    await _db.Insertable(templateAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        #endregion

        #region 配方模版
        public async Task<PageUtil> GetRecipeGroupTemplatePage(RecipeGroupTemplateQueryDto dto)
        {
            var conModels = new List<IConditionalModel>();
            if (!string.IsNullOrEmpty(dto.Name))
                conModels.Add(new ConditionalModel() { FieldName = "Name", ConditionalType = ConditionalType.Like, FieldValue = dto.Name });
            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_RecipeGroupTemplate>()
                .Where(conModels)
                .OrderByDescending(t => t.CreateAt)
               .Select(t => new RecipeGroupTemplateDto
               {
                   Id = t.Id,
                   Name = t.Name,
                   Describe = t.Describe
               }).ToPageListAsync(dto.Current, dto.PageSize, total);

            var util = new PageUtil()
            {
                Total = total,
                Data = res
            };
            return util;
        }

        public async Task<bool> UpdateRecipeGroupTemplate(RecipeGroupTemplateUpDto dto)
        {
            var groupTemplateList = await _db.Queryable<BPA_RecipeGroupTemplate>().ToListAsync();
            if (string.IsNullOrWhiteSpace(dto.Id))
            {
                var same = groupTemplateList.Any(t => t.Name == dto.Name);
                if (same)
                    throw Oops.Oh($"{dto.Name}已存在");
                var template = new BPA_RecipeGroupTemplate
                {
                    Name = dto.Name,
                    Describe = dto.Describe
                };
                await _db.Insertable(template).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
            }
            else
            {
                var template = groupTemplateList.FirstOrDefault(t => t.Id == dto.Id);
                if (template == null)
                    throw Oops.Oh($"{dto.Name}不存在或已删除");
                if (template.Name != dto.Name)
                {
                    var same = groupTemplateList.Any(t => t.Name == dto.Name);
                    if (same)
                        throw Oops.Oh($"{dto.Name}已存在");
                }
                template.Name = dto.Name;
                template.Describe = dto.Describe;
                await _db.Updateable(template).ExecuteCommandAsync();
            }
            return true;
        }

        public async Task<bool> DeleteRecipeGroupTemplate(string ids)
        {
            var templateList = await _db.Queryable<BPA_RecipeGroupTemplate>().ToListAsync();

            return true;
        }
        #endregion
    }
}
