﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.Store;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Core.Base;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase
{
    [ApiDescriptionSettings("Store", Tag = "门店信息")]
    public class StoreServices : IDynamicApiController
    {
        IStoreService _storeService;
        public StoreServices(IStoreService storeService)
        {
            _storeService = storeService;
        }

        /// <summary>
        /// 门店分页查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/store/getStorePage")]
        public async Task<PageUtil> GetStorePage(StoreQueryDto dto)
        {
            return await _storeService.GetStorePage(dto);
        }

        /// <summary>
        /// 获取所有门店
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/store/getAllStoreList")]
        public async Task<List<StoreInfoDto>> GetAllStoreList()
        {
            return await _storeService.GetAllStoreList();
        }

        /// <summary>
        /// 更新门店信息
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/store/updateStore")]
        public async Task<bool> UpdateStore(StoreInfoDto dto)
        {
            return await _storeService.UpdateStore(dto);
        }

        /// <summary>
        /// 删除门店信息
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        [HttpPost("/api/store/deleteStore")]
        public async Task<bool> DeleteStore(List<string> Ids)
        {
            return await _storeService.DeleteStore(Ids);
        }

        /// <summary>
        /// 获取所有设备
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/store/getAllDeviceList")]
        public async Task<List<DeviceInfoDto>> GetAllDeviceList()
        {
            return await _storeService.GetAllDeviceList();
        }

        /// <summary>
        /// 设备分页查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/store/getDevicePage")]
        public async Task<PageUtil> GetDevicePage(DeviceQueryDto dto)
        {
            return await _storeService.GetDevicePage(dto);
        }

        /// <summary>
        /// 更新设备信息
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/store/updateDevice")]
        public async Task<bool> UpdateDevice(DeviceInfoDto dto)
        {
            return await _storeService.UpdateDevice(dto);
        }

        /// <summary>
        /// 删除设备信息
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        [HttpPost("/api/store/deleteDevice")]
        public async Task<bool> DeleteDevice(List<string> Ids)
        {
            return await _storeService.DeleteDevice(Ids);
        }

        /// <summary>
        /// 分页查询设备日志
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/store/getDeviceLogPage")]
        public async Task<PageUtil> GetDeviceLogPage(DeviceLogQueryDto dto)
        {
            return await _storeService.GetDeviceLogPage(dto);
        }
    }
}
