﻿using BPA.SAAS.Manage.Application.Org.Dtos.Role;
using BPA.SAAS.Manage.Application.Org.Interface;
using BPA.SAAS.Manage.Application.System.Dtos;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Comm.Enum;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.Org;
using BPA.SAAS.Manage.Core.System;
using Furion.LinqBuilder;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.Org.Services
{
    public class RoleService: IRoleService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public RoleService(ISqlSugarClient db) 
        {
            _db=db;
        }
        public async Task<PageUtil> Page(RoleDtoPageInput input)
        {
            RefAsync<int> total = 0;
            var res =await _db.Queryable<BPA_Roles>()
                   .WhereIF(!input.Code.IsNullOrEmpty(), t => t.Code.Contains(input.Code))
                   .WhereIF(!input.Name.IsNullOrEmpty(), t => t.Name.Contains(input.Name))

                   .Select(t => new RoleDtoOutput
                   {
                       CreateAt = t.CreateAt,
                       Code = t.Code,
                       Id = t.Id,
                       Name = t.Name,
                       Remark = t.Remark,
                       Sort = t.Sort,
                   }).OrderBy(t => t.Sort).OrderBy(x => x.CreateAt, OrderByType.Desc).ToPageListAsync(input.Current, input.PageSize,  total);
            PageUtil util = new PageUtil()
            {
                Total = total,
                Data = res

            };
            return util;
        }
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Add(RoleDtoInput input)
        {

            //if (input.Code != "sys_manager_role")
            //{
            //    var resEntitites = _db.Queryable<BPA_Roles>().Where(a => a.Code == input.Code).ToList();
            //    if (resEntitites.Count() > 0)
            //    {
            //        throw Oops.Oh("已存在相同编码的角色");
            //    }
            //}
        
            var role = input.Adapt<BPA_Roles>();
            input.Code = GetNumber2();
            // role.DataScopeType = DataScopeType.ALL;
            return await _db.Insertable(role).CallEntityMethod(t => t.Create()).ExecuteCommandAsync() > 0;
        }
        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Update(RoleDtoInput input)
        {
            var BPA_Roles = await _db.Queryable<BPA_Roles>().Where(a => a.Code == input.Code).FirstAsync();
            var code = BPA_Roles.Code;
            if (code != input.Code)
            {
                throw Oops.Oh("角色编码不能进行修改");
            }
            var Roles = await _db.Queryable<BPA_Roles>().Where(a => a.Id == input.Id).FirstAsync();
            Roles.Name= input.Name;
            Roles.Code= input.Code;
            Roles.Remark= input.Remark;
            Roles.Sort = input.Sort;
            return _db.Updateable(Roles).ExecuteCommandHasChange();

        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Delete(List<string> input)
        {
            try
            {
                _db.Ado.BeginTran();
                // 查询数据库中是否存在未删除的活动信息
                var resEntitites = _db.Queryable<BPA_Roles>().In(input).ToList();
                //校验用户
                var dara = await _db.Queryable<BPA_UserRole>().Where(x => input.Contains(x.SysRoleId)).ToListAsync();
                if (!dara.Any())
                {
                    await _db.Deleteable<BPA_RoleMenu>().Where(t => input.Contains(t.SysRoleId)).ExecuteCommandAsync();
                    await _db.Deleteable<BPA_BasicFunctionRoleMenu>().Where(t => input.Contains(t.SysRoleId)).ExecuteCommandAsync();
                    var res =await _db.Deleteable(resEntitites).ExecuteCommandAsync();
                    _db.Ado.CommitTran();
                    return res > 0;
                }
                else
                {
                    throw Oops.Oh("该角色已有用户使用");
                }
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        /// <summary>
        /// 根据角色查询角色菜单
        /// </summary>
        /// <param name="RoleId"></param>
        /// <returns></returns>
        public async Task<List<QueryRoleMenuListDto>> GetMenuRole(string RoleId)
        {
            var MenuName =await _db.Queryable<BPA_RoleMenu>().LeftJoin<BPA_Menu>((t, x) => t.SysMenuId == x.Id)
                  .Where(t => t.SysRoleId == RoleId).OrderBy((t, x) => x.CreateAt).Select((t, x) => new QueryRoleMenuListDto()
                  {
                      Name = x.Name,
                      Id = x.Id,
                      IsParentMenu = string.IsNullOrWhiteSpace(x.Pid)
                  }).ToListAsync();

            return MenuName;
        }

        /// <summary>
        /// 根据角色查询基础功能菜单
        /// </summary>
        /// <param name="roleId"></param>
        /// <returns></returns>
        public async Task<List<BasicMenuDto>> GetBasicFunctionList(string roleId)
        {
            var menuList = await _db.Queryable<BPA_BasicFunctionMenu>().ToListAsync();
            var result = await _db.Queryable<BPA_BasicFunctionRoleMenu>().LeftJoin<BPA_BasicFunctionMenu>((a, b) => a.BasicFunctionId == b.Id)
                .Where((a, b) => a.SysRoleId == roleId)
                .OrderBy((a, b) => b.Sort)
                .Select((a, b) => new BasicMenuDto
                {
                    Id = b.Id,
                    Name = b.Name,
                    IsParentMenu = menuList.Any(t=>t.Pid == b.Id)
                }).ToListAsync();
            return result;
        }

        /// <summary>
        /// 更新基础功能授权
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateBasicFunction(BasicFunctionMenuDataDto dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var data = new List<BPA_BasicFunctionRoleMenu>();
                var menuIdList = new List<string>();
                await _db.Deleteable<BPA_BasicFunctionRoleMenu>(t => t.SysRoleId == dto.SysRoleId).ExecuteCommandAsync();
                if(dto.BasicMenuIdList.Any())
                {
                    var menuList = await _db.Queryable<BPA_BasicFunctionMenu>().ToListAsync();
                    foreach (var item in dto.BasicMenuIdList)
                    {
                        var pid = menuList.FirstOrDefault(t => t.Id == item).Pid;
                        if (pid != null)
                        {
                            var pid2 = menuList.FirstOrDefault(t => t.Id == pid).Pid;
                            if (pid2 != null)
                            {
                                var pid3 = menuList.FirstOrDefault(t => t.Id == pid2).Pid;
                                if (pid3 != null)
                                    menuIdList.Add(pid3);
                                menuIdList.Add(pid2);
                            }
                            menuIdList.Add(pid);
                        }
                        menuIdList.Add(item);
                    }
                    menuIdList.Distinct().ToList().ForEach(t =>
                    {
                        data.Add(new BPA_BasicFunctionRoleMenu
                        {
                            Id = Guid.NewGuid().ToString(),
                            SysRoleId = dto.SysRoleId,
                            BasicFunctionId = t
                        });
                    });
                }
                await _db.Insertable(data).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 获取用户基础功能权限编码
        /// </summary>
        /// <returns></returns>
        public async Task<List<string>> GetBasicRoleCode()
        {
            var result = new List<string>();
            var userId = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
            var sysRoleIdList = await _db.Queryable<BPA_UserRole>().Where(t => t.SysUserId == userId).Select(a => a.SysRoleId).ToListAsync();
            var basicFunctionIdList = await _db.Queryable<BPA_BasicFunctionRoleMenu>().Where(t => sysRoleIdList.Contains(t.SysRoleId)).Select(a => a.BasicFunctionId).Distinct().ToListAsync();
            var menuList = await _db.Queryable<BPA_BasicFunctionMenu>().ToListAsync();
            foreach (var id in basicFunctionIdList)
            {
                var menuCode = menuList.FirstOrDefault(t => t.Id == id)?.Code;
                if(menuCode != null)
                    result.Add(menuCode);
            }
            return result;
        }

        /// <summary>
        /// 编辑角色菜单
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> AddOrUpdateMenuRole(RoleMenuDtoInput input)
        {
            List<BPA_RoleMenu> models = new List<BPA_RoleMenu>();
            List<string> NewMenuId = new List<string>();
            _db.Deleteable<BPA_RoleMenu>(t => t.SysRoleId == input.SysRoleId).ExecuteCommandHasChange();
            if (input.SysMenuId != null)
            {
                var Menus = _db.Queryable<BPA_Menu>().ToList();

                foreach (var item in input.SysMenuId)
                {
                    var Pid = Menus.FirstOrDefault(t => t.Id == item).Pid;
                    if (Pid != null && Pid != "")
                    {
                        var Pid2 = Menus.FirstOrDefault(t => t.Id == Pid).Pid;

                        if (Pid2 != null)
                        {
                            NewMenuId.Add(Pid2);
                        }
                        NewMenuId.Add(Pid);
                    }
                    NewMenuId.Add(item);
                }
                NewMenuId.Distinct().ToList().ForEach(item =>
                {
                    models.Add(new BPA_RoleMenu
                    {
                        Id = Guid.NewGuid().ToString(),
                        SysMenuId = item,
                        SysRoleId = input.SysRoleId
                    });
                });
            }
            if (models.Count <= 0)
                return true;

           
            return await _db.Insertable(models).ExecuteCommandAsync() > 0;


        }
        /// <summary>
        /// 启用
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<bool> Enable(string Id)
        {
            var resEntitites = _db.Queryable<BPA_Roles>().In(Id).ToList();
            var res =await _db.Updateable(resEntitites).ExecuteCommandAsync();
            return res>0;
        }

        /// <summary>
        /// 禁用
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<bool> Disable(string Id)
        {
            var resEntitites = _db.Queryable<BPA_Roles>().In(Id).ToList();
            var res =await _db.Updateable(resEntitites).ExecuteCommandAsync();
            return res>0;
        }
        public async Task<Dictionary<string, object>> GetRole()
        {
            var res =await _db.Queryable<BPA_Roles>().ToDictionaryAsync(t => t.Id, t => t.Name);
            return res;
        }
        private string GetNumber2(int Length = 10)
        {
            byte[] buffer = Guid.NewGuid().ToByteArray();
            var ram = BitConverter.ToInt64(buffer, 0);
            var str = string.Format("{0}", ram.ToString().Substring(0, Length));
            return str;
        }
    }
}
